/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __DB_H
#define __DB_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <iosfwd>
#include <string>
//#include <sqlplus.hh>
#include <exception>
#include <dbi/dbi.h>
#include <vector>
#include <list>
#include "triplet.h"
#include "wlmodule.h"
#include "signals.h"
//#include "dbdefs.h"

/**
	@brief DB deals with the database backend
	@author Michael Moritz
*/
class DB : public Event_Handler
{
public:
	typedef std::vector<std::string> ParamVector;
	DB() throw(std::exception);
	
	///	enum of record (triplet) states
	typedef enum { 
		unknown, /**< triplet is unknown, has been added to db with time and count 0 */
		waiting, /**< triplet is known, time it has been in db is less timeout and count = 0 */
		registered /**< triplet is registered, count > 1 and/or time > timeout */
	} RecordStatus;
	RecordStatus update(const class Triplet&) throw(std::exception); 
	
	///	list installed drivers (see http://dbi.sf.net for more drivers)
	void list_drivers(std::ostream &os);
	
	~DB() { close(); }
	
	///	@returns a string with error msg from dbi 
	std::string getConError(const std::string&) const;
	
	/// create a table if it doesnt exist
	void createTable(const class WLStaticDef& staticDef) throw(std::exception);
	
	bool exactMatch(const std::string& logprefix,const class WLStaticDef& staticDef,
		const class Triplet& triplet) throw(std::exception);
		
	dbi_result executeSQL(const std::string& sql) throw(std::exception);
		
// 	enum { tempCharBuf = 255 };
	
	/// signal handling
	  // Hook method.
	virtual int handle_signal(int signum);
protected:
	const std::string trimLastByte(std::string);
	std::string getSqlCondition(const WLCompares& compares,const Triplet& triplet) const;
	const std::string numToStr(unsigned) const;
	std::string getWeakHost(const std::string&) const throw(std::exception);
	
	/// open a db connection
	void open() throw(std::exception);
	
	///	destroy db connection 
	void close();
	
	/// initialise wl modules based on config
	void init_wlMods();
private:
	dbi_conn _con;
	static const std::string _sql_table_test;
	typedef std::list<WlModule*>WlModuleList;
	WlModuleList _wlMods;
	sig_atomic_t _graceful_quit;
};

#endif

