/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <cassert>
#include "dbdefs.h"
#include "defs.h"


///mysql commands and definitions for whitelisting modules
static const WLStaticDef g_wlSqlDefs[] = { 

	{ TABLE_NETWORK,
		//testTable statement: this is used for testing if table exists
		//                       and for initialising the memory cache
		//						testTable adds WHERE 1=0 to this
		"SELECT * FROM "TABLE_NETWORK,
		//createTable: this is used in case test fails to create the table
		{ "CREATE TABLE "TABLE_NETWORK" ( "
			ATTR_NWADDRESS" VARCHAR("STRINGIFY(MAX_LEN_NWADDRESS)") not null default '',"
			ATTR_NWCOMMENT" VARCHAR("STRINGIFY(MAX_LEN_NWCOMMENT)") DEFAULT ''," },
		//the field used for PRIMARY KEY 
		ATTR_NWADDRESS,
		WlModule::CM_exactMatch, 
		"SELECT "ATTR_NWCOMMENT" FROM "TABLE_NETWORK,
		{
			{ ATTR_NWADDRESS,"=", Triplet::TM_quotedweakclient,0},
//example			{ ATTR_NWADDRESS,"=", Triplet::TM_weakclient,"AND"},
			{ NULL }
		},
		// comment field (only used by dbcached)
		ATTR_NWCOMMENT
	},
	{ TABLE_RECIPIENT,
		"SELECT * FROM "TABLE_RECIPIENT,
		{ "CREATE TABLE "TABLE_RECIPIENT" ( "
			ATTR_RCPTADDRESS" VARCHAR("STRINGIFY(MAX_LEN_RCPTADDRESS)") NOT NULL DEFAULT '',"
			ATTR_RCPTCOMMENT" VARCHAR("STRINGIFY(MAX_LEN_RCPTCOMMENT)") DEFAULT ''," },
		ATTR_RCPTADDRESS,
		WlModule::CM_exactMatch, 
		"SELECT "ATTR_RCPTCOMMENT" FROM "TABLE_RECIPIENT,
		{
			{ ATTR_RCPTADDRESS,"=", Triplet::TM_recipient,0},
			{ NULL }
		},
		ATTR_RCPTCOMMENT
	},
	{ TABLE_SENDER,
		"SELECT * FROM "TABLE_SENDER,
		{ "CREATE TABLE "TABLE_SENDER" ( "
			ATTR_SNDADDRESS" VARCHAR("STRINGIFY(MAX_LEN_SNDADDRESS)") NOT NULL DEFAULT '',"
			ATTR_SNDCOMMENT" VARCHAR("STRINGIFY(MAX_LEN_SNDCOMMENT)") DEFAULT ''," },
		ATTR_SNDADDRESS,
		WlModule::CM_exactMatch, 
		"SELECT "ATTR_SNDCOMMENT" FROM "TABLE_SENDER,
		{
			{ ATTR_SNDADDRESS,"=", Triplet::TM_sender,0},
			{ NULL }
		},
		ATTR_SNDCOMMENT
	},
	{ TABLE_PATTERN,
		"SELECT * FROM "TABLE_PATTERN,
		{ "CREATE TABLE "TABLE_PATTERN" ( "
			ATTR_EXPRESSION" VARCHAR("STRINGIFY(MAX_LEN_EXPRESSION)") NOT NULL DEFAULT '',"
			ATTR_PATTCOMMENT" VARCHAR("STRINGIFY(MAX_LEN_PATTCOMMENT)") DEFAULT ''," },
		ATTR_EXPRESSION,
		WlModule::CM_patternMatch, 
		"SELECT * FROM "TABLE_PATTERN,
		{
			{ NULL }
		},
		ATTR_PATTCOMMENT
	},
	{ NULL }
};
///mysql commands and definitions for the triplets table
static const WLStaticDef g_tripletsSqlDef =
{ 	TABLE_TRIPLETS,
		"SELECT * FROM "TABLE_TRIPLETS,
		{	"CREATE TABLE "TABLE_TRIPLETS" ("
				ATTR_CLIENT" VARCHAR("STRINGIFY(MAX_LEN_CLIENT)"),"
				ATTR_SENDER" VARCHAR("STRINGIFY(MAX_LEN_SENDER)") NOT NULL,"
				ATTR_RECIPIENT" VARCHAR("STRINGIFY(MAX_LEN_RECIPIENT)") NOT NULL,"
				TRIPLETS_IPDEF(64)" NUMERIC(4,0) DEFAULT 0 NOT NULL,"
				TRIPLETS_IPDEF(32)" NUMERIC(4,0) DEFAULT 0 NOT NULL,"
				TRIPLETS_IPDEF(16)" NUMERIC(4,0) DEFAULT 0 NOT NULL,"
				TRIPLETS_IPDEF(8)" NUMERIC(4,0) DEFAULT 0 NOT NULL,"
				"count INT NOT NULL DEFAULT '0',"
				"uts INT NOT NULL,"
			 	"PRIMARY KEY ("
					ATTR_RECIPIENT","ATTR_SENDER
					","TRIPLETS_IPDEF(64)","TRIPLETS_IPDEF(32)","TRIPLETS_IPDEF(16)","TRIPLETS_IPDEF(8)
					")",
			"CREATE INDEX sender_recipient_index ON "TABLE_TRIPLETS
				"("ATTR_SENDER"(15),"ATTR_RECIPIENT"(15)"
		}
};

const WLStaticDef& g_getTripletDef()
{
	return g_tripletsSqlDef;
}
const WLStaticDef& g_getWlDef(const std::string& tableName) throw (std::out_of_range)
{
	unsigned iSql = 0;
	while((iSql < ((sizeof(g_wlSqlDefs)/sizeof(WLStaticDef))-1))
		&& ::g_wlSqlDefs[iSql]._tableName ) {
		if(::g_wlSqlDefs[iSql]._tableName == tableName) 
			return ::g_wlSqlDefs[iSql];
		iSql++;
	}
	throw std::out_of_range("Possible (typing) error in configuration. No whitelisting supported for: "
		+tableName+" (g_getWlDef)");
}
