/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __DBIQUOTE_H
#define __DBIQUOTE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <stdexcept>

#define DBIQUOTEDBG false

//very basic search/replace function
template <typename T>
	std::string strEscape(T in,const char search, const char esc, size_t size)
	{
		std::string out = "";
		for(size_t i=0;i < size; i++) {
			const char c = in[i];
			if(c == search)
				out += esc;
			out += c;
		}
		//std::cout << out << endl;
		return out;
	}
	std::string strEscape(const std::string &in,const char search, const char esc)
	{
		return strEscape<std::string>(in, search, esc, in.size());
	}
// two convenience macros
/// escape a printf-type statement, replace any % with %%
/// used for syslog and DBI_CON_QUERYF calls
#define ESC_LOG(s) strEscape(s,'%','%')
/// escape a sql string, prefix apostrophe with a backslash
#define ESC_SQL(s) strEscape(s,'\'','\\')

#ifdef OLD_DBI_VERSION 
#define DBI_CON_QUERYF dbi_conn_query

#warning "using old DBI dbi_driver_quote_string -> using workaround"
/// this is a workaround for old versions of libdbi (pre 0.72)
/// that dont implement dbi_quote_string correctly
class DBIQuote
{
	///helper escaping special characters in sql queries
	std::string escapeSQL(const std::string &str) const {
		std::string strRet;
		for(unsigned i=0; i < str.size(); i++) {
			if(str[i] == '\'')
				strRet += "\\";
			strRet += str[i];
		}
		return strRet;
	}
public:
	DBIQuote(dbi_driver,const std::string& str) throw (std::runtime_error)
	: _str("'" + ESC_SQL(str) + "'") {
		if(DBIQUOTEDBG)
			std::cout << str << "->" << _str <<std::endl;
	}
	const char* operator() () { return _str.c_str(); }
	DBIQuote& operator=(const DBIQuote& o) {
		_str = o._str;
		return *this;
	}
	void setNULL() { _str = "NULL"; }
	bool isNULL() const { return (_str == "NULL"); }
private:
	std::string _str;
};

#else
#define DBI_CON_QUERYF dbi_conn_queryf
class DBIQuote
{
public:
	DBIQuote(dbi_driver drv,const std::string& str) throw (std::bad_alloc,std::runtime_error)
	{
//		_psz = (char*) malloc((str.length()+1)*sizeof(char));
//		_psz = (char*) malloc( ((str.length()*2)+4+1)  * sizeof(char));
		dbi_driver_quote_string_copy(drv,str.c_str(),&_psz);
		if(!_psz)
			throw std::runtime_error("dbi_driver_quote_string_copy "+str);
		if(DBIQUOTEDBG)
			std::cout << str << "->" << _psz <<std::endl;
	}
	const char* operator() () { return _psz; }
	DBIQuote& operator=(const DBIQuote& o) throw (std::bad_alloc,std::runtime_error){
		_psz = strdup(o._psz);
		if(!_psz)
			throw std::runtime_error("DBIQuote& operator= strdup");
		return *this;
	}
	~DBIQuote() 
	{
		if(_psz) { //always
			free(_psz);
			_psz = 0;
		}
	}
	void setNULL() throw (std::bad_alloc,std::runtime_error) { 
		if(_psz) {
			free(_psz);
			_psz = 0;
		}
		_psz = strdup("NULL");
		if(!_psz)
			throw std::runtime_error("setNULL strdup");
	}
	bool isNULL() { return (strcmp(_psz,"NULL") == 0); } //TODO can _psz be NULL
private:
	char *_psz;
};

#endif

#endif //__DBIQUOTE_H
