/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef __DEFS_H_
#define __DEFS_H_

/** @file defs.h common definitions 
*/


#ifdef STRINGIFY
///this is maybe defined in system libraries
#warning "STRINGIFY already defined somewhere else, expect problems" 
#undef STRINGIFY
#endif
/// macro to do less macros 
/// turn a numeric def into a text
#define STRINGIFY(arg) _XY_str__(arg)
#define _XY_str__(s)  #s

#define TRIPLETS_IPDEF(s) "ip"STRINGIFY(s)

/// Uncomment this if you want support for the old db scheme
/// in which client ip addresses are stored as strings
/// The new scheme stores them as 4 bytes
//#define OLDWEAK (true)

///currently number of bytes in an ip address
/// more general, number of elements in ip address
/// @see db.cpp 
#define IPADDR_NUM		4

/** Triplets database column names
*/
#define TABLE_TRIPLETS	"triplet"

/** Attributes passed by postfix
	http://www.postfix.org/SMTPD_POLICY_README.html#protocol
	TODO version 2.2 of postfix also support sasl information
*/
#define ATTR_CLIENT 	"client_address"
#define ATTR_SENDER 	"sender"
#define ATTR_RECIPIENT  "recipient"
#define ATTR_RCLNAME	"reverse_client_name"
/** significant part of input strings 
 *	mysql limits INDEX size to 500 
**/
#define MAX_LEN_CLIENT 40
#define MAX_LEN_SENDER 160
#define MAX_LEN_RECIPIENT 160
#endif

/** network database column names
*/
#define TABLE_NETWORK	"network"
#define ATTR_NWADDRESS	"address"
#define ATTR_NWCOMMENT	"comment"

#define MAX_LEN_NWADDRESS 16
#define MAX_LEN_NWCOMMENT 30

/** recipient database column names
*/
#define TABLE_RECIPIENT		"recipient"
#define ATTR_RCPTADDRESS	"address"
#define ATTR_RCPTCOMMENT	"comment"

#define MAX_LEN_RCPTADDRESS 200
#define MAX_LEN_RCPTCOMMENT 30

/** sender database column names
*/
#define TABLE_SENDER		"sender"
#define ATTR_SNDADDRESS		"address"
#define ATTR_SNDCOMMENT		"comment"

#define MAX_LEN_SNDADDRESS 200
#define MAX_LEN_SNDCOMMENT 30

/** pattern database column names
*/
#define TABLE_PATTERN		"pattern"
#define ATTR_EXPRESSION		"expression"
#define ATTR_PATTCOMMENT	"comment"

#define MAX_LEN_EXPRESSION 200
#define MAX_LEN_PATTCOMMENT 30

/**
* when to reset the counter to 1 
*/
#define COUNT_RESET 32768

/**
  * number of max compare operations for wl
**/
#define MAX_WL_COMPARES 3

/**
 * max number of statements that can be used 
 * to create a table and indexes in SQL 
 * @see DB::createTable @see dbdefs.h @see dbdefs.cpp
*/
#define MAX_CREATECMDS 3


