/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "read.h"
#include <iostream>
#include <string>
#include "cfg.h"

using namespace std;

const string Reader::_s_notfound = "key not found";

int Reader::parse(const Reader::AttributeSet& attr, istream &is) 
{
	while(!is.eof()) {
		string line;
		getline(is,line);	
		if(line.length() > 0) {
			if( g_getCfg().isVerbose() ) // only build inpu string if in verbose mode
				_input += line + " ";
			string::size_type eqpos = line.find('=');
			if(eqpos != string::npos) {
				const string key = line.substr(0,eqpos);
				Reader::AttributeSet::const_iterator itr = attr.find(key);
				if( itr != attr.end() ) {
					const string val = line.substr(eqpos+1,string::npos);
					_map.insert(AttributeMap::value_type(key,val));
					//cout << "key: " << key << " val: "<<val<< endl;
				}
			}
		} else {
			return _map.size();
		}
	}
	return -1;
}
bool Reader::haveAllRequired(const Reader::AttributeSet& attr) const
{
	AttributeSet::const_iterator itr;
	for(itr=attr.begin(); itr != attr.end(); ++itr) {
		if( ! itr->second) 
			continue;
		if( _map.find(itr->first) == _map.end() )
			return false;
	}
	return true;
}



/*
int main()
{
	Reader::AttributeSet attrSet;
	attrSet.insert("client_address");
	attrSet.insert("sender");
	attrSet.insert("recipient");

	Reader testReader(attrSet,cin);
}
*/
