/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __REGEX_H
#define __REGEX_H
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <exception>
#include <string>
#include <regex.h> /* Provides regular expression matching */

class RegEx
{
public: 
	RegEx(int cflags=REG_ICASE|REG_NOSUB|REG_EXTENDED) //removed REG_NEWLINE|
	: 	m_regex(0),
		m_compflags(cflags)
	{}
	virtual ~RegEx() { freeRegex(); }
	void setPattern(const std::string& s) throw(std::bad_alloc, std::exception)
	{
		/* Make space for the regular expression */
		freeRegex(); 
		initRegex();
		int err_no;
		if((err_no=regcomp(m_regex, s.c_str(), m_compflags)) != 0) /* Compile the regex */
		{
      		size_t length;
      		char *buffer;
			std::string strErr;
      		length = regerror (err_no, m_regex, NULL, 0);
      		buffer = new char[length];
      		regerror (err_no, m_regex, buffer, length);
			strErr = buffer;
      		delete buffer;
      		throw std::runtime_error("Regex: " + strErr);
    	}
	}
	bool match(const std::string &s) const
	{
		size_t no_sub = m_regex->re_nsub+1; /* How many matches are there in a line? */
		regmatch_t *result = 0;
		if(regexec(m_regex, s.c_str(), no_sub, result, 0) == 0) /* Found a match */
			return true;
		return false;
	}
protected:
	void freeRegex() {
		if(m_regex) {
			regfree( m_regex );
			delete m_regex;
			m_regex = 0;
		}
	}
	void initRegex() {
		m_regex = new regex_t;
		memset(m_regex, 0, sizeof(regex_t));
	}
private:
	regex_t *m_regex;
  	int m_compflags;
};

#endif
