/***************************************************************************
 *   Copyright (C) 2004 by Michael Moritz                                  *
 *   mimo@restoel.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef WLCACHEDDB_H
#define WLCACHEDDB_H

#include <exception>
#include <string>
#include <map>
#include "triplet.h"
#include "db.h"
#include "wlmodule.h"

/**
@brief whitelisting module that reads the data stored and caches it in memory
@author Michael Moritz
*/
class WlCachedDB
{
	class Heading : public std::map<std::string, unsigned>
	{
	};
	class Entry : public std::map<unsigned, std::string>
	{
	};
	class Data : public std::map<std::string, Entry>
	{
	};
public:
    WlCachedDB(class DB& db,
		bool initmode,
		const WLStaticDef& staticDef) throw(std::exception);
	bool check(const class Triplet& triplet) throw(std::exception);
    ~WlCachedDB();
protected:
	bool exactMatch(const std::string logprefix,const class Triplet& triplet);
	bool patternMatch(const std::string logprefix,const class Triplet& triplet);
	std::string getRowComment(Data::const_iterator& itr) const;
	void logMatch(const std::string& logprefix,Data::const_iterator& itr,const class Triplet& triplet) const;
private:
	class DB& _db;	
	const WLStaticDef& _staticDef;
	Heading _heading;
	Data _data;
};
///helper for uppercase to lowercase transform
struct ToLower
{
     char operator() (char c) const  { return std::tolower(c); }
};
#endif
